# -*- coding: utf-8 -*-

# Copyright (c) 2020, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function
__metaclass__ = type


import base64

from ansible.module_utils.common.text.converters import to_native, to_bytes

from ansible_collections.community.crypto.plugins.plugin_utils.action_module import ActionModuleBase

from ansible_collections.community.crypto.plugins.module_utils.crypto.basic import (
    OpenSSLObjectError,
)

from ansible_collections.community.crypto.plugins.module_utils.crypto.module_backends.privatekey import (
    select_backend,
    get_privatekey_argument_spec,
)


class PrivateKeyModule(object):
    def __init__(self, module, module_backend):
        self.module = module
        self.module_backend = module_backend
        self.check_mode = module.check_mode
        self.changed = False
        self.return_current_key = module.params['return_current_key']

        if module.params['content'] is not None:
            if module.params['content_base64']:
                try:
                    data = base64.b64decode(module.params['content'])
                except Exception as e:
                    module.fail_json(msg='Cannot decode Base64 encoded data: {0}'.format(e))
            else:
                data = to_bytes(module.params['content'])
            module_backend.set_existing(data)

    def generate(self, module):
        """Generate a keypair."""

        if self.module_backend.needs_regeneration():
            # Regenerate
            if not self.check_mode:
                self.module_backend.generate_private_key()
                privatekey_data = self.module_backend.get_private_key_data()
                self.privatekey_bytes = privatekey_data
            else:
                self.module.deprecate(
                    'Check mode support for openssl_privatekey_pipe will change in community.crypto 3.0.0'
                    ' to behave the same as without check mode. You can get that behavior right now'
                    ' by adding `check_mode: false` to the openssl_privatekey_pipe task. If you think this'
                    ' breaks your use-case of this module, please create an issue in the'
                    ' community.crypto repository',
                    version='3.0.0',
                    collection_name='community.crypto',
                )
            self.changed = True
        elif self.module_backend.needs_conversion():
            # Convert
            if not self.check_mode:
                self.module_backend.convert_private_key()
                privatekey_data = self.module_backend.get_private_key_data()
                self.privatekey_bytes = privatekey_data
            else:
                self.module.deprecate(
                    'Check mode support for openssl_privatekey_pipe will change in community.crypto 3.0.0'
                    ' to behave the same as without check mode. You can get that behavior right now'
                    ' by adding `check_mode: false` to the openssl_privatekey_pipe task. If you think this'
                    ' breaks your use-case of this module, please create an issue in the'
                    ' community.crypto repository',
                    version='3.0.0',
                    collection_name='community.crypto',
                )
            self.changed = True

    def dump(self):
        """Serialize the object into a dictionary."""
        result = self.module_backend.dump(include_key=self.changed or self.return_current_key)
        result['changed'] = self.changed
        return result


class ActionModule(ActionModuleBase):
    @staticmethod
    def setup_module():
        argument_spec = get_privatekey_argument_spec()
        argument_spec.argument_spec.update(dict(
            content=dict(type='str', no_log=True),
            content_base64=dict(type='bool', default=False),
            return_current_key=dict(type='bool', default=False),
        ))
        return argument_spec, dict(
            supports_check_mode=True,
        )

    @staticmethod
    def run_module(module):
        backend, module_backend = select_backend(
            module=module,
            backend=module.params['select_crypto_backend'],
        )

        try:
            private_key = PrivateKeyModule(module, module_backend)
            private_key.generate(module)
            result = private_key.dump()
            if private_key.return_current_key:
                # In case the module's input (`content`) is returned as `privatekey`:
                # Since `content` is no_log=True, `privatekey`'s value will get replaced by
                # VALUE_SPECIFIED_IN_NO_LOG_PARAMETER. To avoid this, we remove the value of
                # `content` from module.no_log_values. Since we explicitly set
                # `module.no_log = True`, this should be safe.
                module.no_log = True
                try:
                    module.no_log_values.remove(module.params['content'])
                except KeyError:
                    pass
                module.params['content'] = 'ANSIBLE_NO_LOG_VALUE'
            module.exit_json(**result)
        except OpenSSLObjectError as exc:
            module.fail_json(msg=to_native(exc))



#!/usr/bin/python
#
# Copyright (c) 2022 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_sqlelasticpool
version_added: "1.14.0"
short_description: Manage SQL Elastic Pool instance
description:
    - Create, update and delete instance of SQL Elastic Pool.

options:
    resource_group:
        description:
            - The name of the resource group that contains the resource.
            - You can obtain this value from the Azure Resource Manager API or the portal.
        required: True
        type: str
    server_name:
        description:
            - The name of the server.
        required: True
        type: str
    name:
        description:
            - The name of the elastic pool to be operated on (updated or created).
        required: True
        type: str
    location:
        description:
            - Resource location. If not set, location from the resource group will be used as default.
        type: str
    sku:
        description:
            - The sku of the elastic pool. The Elastic PoolEditions enumeration contains all the valid sku.
        type: dict
        suboptions:
            name:
                description:
                    - Name of the elastic pool SKU, typically, a letter + Number code, e.g. P3
                required: True
                type: str
            tier:
                description:
                    - The tier or edition of the particular SKU, e.g. Basic, Premium
                type: str
            capacity:
                description:
                    - Capacity of the particular SKU.
                type: int
            size:
                description:
                    - Size of the particular SKU
                type: str
            family:
                description:
                    - If the service has different generations of hardware, for the same SKU, then that can be used here
                type: str
    max_size_bytes:
        description:
            - The max size of the elasticpool expressed in bytes.
            - If not I(create_mode=default), this value is ignored.
            - To see possible values, query the capabilities API (/subscriptions/{subscriptionId}/providers/Microsoft.Sql/locations/{locationID}/capabilities).
              referred to by operationId:'Capabilities_ListByLocation'.
        type: str
    zone_redundant:
        description:
            - Is this elasticpool is zone redundant? It means the replicas of this elasticpool will be spread across multiple availability zones.
        type: bool
        default: False
    per_elasticpool_settings:
        description:
            - The per database settings for the elastic pool.
        type: dict
        suboptions:
            min_capacity:
                description:
                    - The minimum capacity all databases are guaranteed.
                type: float
            max_capacity:
                description:
                    - The maximum capacity all databases are guaranteed.
                type: float
    license_type:
        description:
            - The license type to apply for this elastic pool.
        type: str
        default: LicenseIncluded
        choices:
            - LicenseIncluded
            - BasePrice
    maintenance_configuration_id:
        description:
            - Maintenance configuration id assigned to the elastic pool.
        type: str
    state:
        description:
            - Assert the state of the SQL Elastic Pool. Use C(present) to create or update an SQL Elastic Pool and C(absent) to delete it.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - xuzhang3 (@xuzhang3)
    - Fred Sun (@Fred-sun)
'''

EXAMPLES = '''
- name: Create (or update) SQL Elastic Pool
  azure_rm_elastic_pool:
    resource_group: myResourceGroup
    server_name: sqlcrudtest-5961
    name: testEP
    zone_redundant: false
    sku:
      name: GP_Gen5
      family: Gen5
      capacity: 3
    tags:
      key1: value1

- name: Delete SQL Elastic Pool
  azure_rm_elastic_pool:
    resource_group: myResourceGroup
    server_name: sqlcrudtest-5961
    name: testEP
    state: absent
'''

RETURN = '''
elastic_pool:
    description:
        - A list of dictionaries containing facts for SQL Elastic Pool.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID.
            returned: always
            type: str
            sample: /subscriptions/xxx-xxx/resourceGroups/testrg/providers/Microsoft.Sql/servers/sqlsrvfredsqldb/elasticPools/fedelastic01
        name:
            description:
                - Elastic Pool name.
            returned: always
            type: str
            sample: testEP
        location:
            description:
                - Resource location.
            returned: always
            type: str
            sample: eastus
        tags:
            description:
                - Resource tags.
            returned: always
            type: dict
            sample: { 'taga':'aaa', 'tagb':'bbb' }
        sku:
            description:
                - The name and tier of the SKU.
            returned: always
            type: complex
            contains:
                name:
                    description:
                        - The name of the SKU.
                    returned: always
                    type: str
                    sample: GP_Gen5
                tier:
                    description:
                        - The SKU tier.
                    returned: always
                    type: str
                    sample: GeneralPurpose
                capacity:
                    description:
                        - The SKU capacity.
                    returned: always
                    type: int
                    sample: 2
                family:
                    description:
                        - If the service has different generations of hardware, for the same SKU, then that can be captured here.
                    type: str
                    returned: always
                    sample: Gen5
                size:
                    description:
                        - Size of the particular SKU.
                    type: str
                    returned: always
                    sample: null
        zone_redundant:
            description:
                - Whether or not this database is zone redundant, which means the replicas of this database will be spread across multiple availability zones.
            returned: always
            type: bool
            sample: true
        license_type:
            description:
                - The license type to apply for this elastic pool.
            type: str
            returned: always
            sample: LicenseIncluded
        maintenance_configuration_id:
            description:
                -  Maintenance configuration id assigned to the elastic pool.
            type: str
            returned: always
            sample: /subscriptions/xxx-xxx/providers/Microsoft.Maintenance/publicMaintenanceConfigurations/SQL_Default
        max_size_bytes:
            description:
                - The storage limit for the database elastic pool in bytes.
            type: str
            returned: always
            sample: 34359738368
        per_database_settings:
            description:
                - The per database settings for the elastic pool.
            type: complex
            returned: always
            contains:
                min_capacity:
                    description:
                        - The minimum capacity all databases are guaranteed
                    type: float
                    returned: always
                    sample: 0.0
                max_capacity:
                    description:
                        - The maximum capacity any one database can consume.
                    type: float
                    returned: always
                    sample: 2.0
'''

import time
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError
    from azure.core.polling import LROPoller
except ImportError:
    # This is handled in azure_rm_common
    pass

sku_spec = dict(
    name=dict(type='str', required=True),
    tier=dict(type='str'),
    size=dict(type='str'),
    family=dict(type='str'),
    capacity=dict(type='int')
)


per_elasticpool_settings_spec = dict(
    min_capacity=dict(type='float'),
    max_capacity=dict(type='float')
)


class Actions:
    NoAction, Create, Update, Delete = range(4)


class AzureRMSqlElasticPool(AzureRMModuleBase):
    """Configuration class for an Azure RM SQL Elastic Pool resource"""

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            server_name=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True
            ),
            location=dict(
                type='str'
            ),
            sku=dict(
                type='dict',
                options=sku_spec
            ),
            max_size_bytes=dict(
                type='str'
            ),
            zone_redundant=dict(
                type='bool',
                default=False
            ),
            per_elasticpool_settings=dict(
                type='dict',
                options=per_elasticpool_settings_spec
            ),
            maintenance_configuration_id=dict(
                type='str'
            ),
            license_type=dict(
                type='str',
                default="LicenseIncluded",
                choices=["LicenseIncluded", "BasePrice"]
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )

        self.resource_group = None
        self.server_name = None
        self.name = None
        self.location = None
        self.sku = None
        self.max_size_bytes = None
        self.zone_redundant = None
        self.per_elasticpool_settings = None
        self.maintenance_configuration_id = None
        self.body = dict()

        self.results = dict(changed=False)
        self.to_do = Actions.NoAction

        super(AzureRMSqlElasticPool, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                    supports_check_mode=True,
                                                    supports_tags=True)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])
            if key not in ['resource_group', 'server_name', 'name', 'state']:
                self.body[key] = kwargs[key]

        resource_group = self.get_resource_group(self.resource_group)
        if not self.location:
            # Set default location
            self.location = resource_group.location

        old_response = None
        response = None

        old_response = self.get_elastic_pool()

        if not old_response:
            self.log("SQL Elastic Pool instance doesn't exist")
            if self.state == 'absent':
                self.log("Old instance didn't exist")
            else:
                self.to_do = Actions.Create
        else:
            self.log("SQL Elastic Pool instance already exists")
            if self.state == 'absent':
                self.to_do = Actions.Delete
            elif self.state == 'present':
                self.log(
                    "Need to check if SQL Elastic Pool instance has to be deleted or may be updated")
                if self.per_elasticpool_settings is not None and (self.body['per_elasticpool_settings'] != old_response['per_elasticpool_settings']):
                    self.to_do = Actions.Update
                if self.maintenance_configuration_id and (self.body['maintenance_configuration_id'] != old_response['maintenance_configuration_id']):
                    self.to_do = Actions.Update
                if self.license_type is not None and (self.body['license_type'] != old_response['license_type']):
                    self.to_do = Actions.Update
                if self.zone_redundant is not None and (bool(self.body['zone_redundant']) != bool(old_response['zone_redundant'])):
                    self.to_do = Actions.Update
                if self.max_size_bytes is not None and (self.body['max_size_bytes'] != old_response['max_size_bytes']):
                    self.to_do = Actions.Update
                if self.sku is not None:
                    for key in self.sku.keys():
                        if self.sku[key] is not None and (self.sku[key] != old_response['sku'][key]):
                            self.to_do = Actions.Update
                        else:
                            self.sku[key] = old_response['sku'][key]
                update_tags, newtags = self.update_tags(old_response.get('tags', dict()))
                if update_tags:
                    self.body['tags'] = newtags
                    self.to_do = Actions.Update

        if (self.to_do == Actions.Create) or (self.to_do == Actions.Update):
            self.log("Need to Create / Update the SQL Elastic Pool instance")

            self.results['changed'] = True

            if self.check_mode:
                return self.results

            if self.to_do == Actions.Create:
                self.body['location'] = self.location
                response = self.create_elastic_pool(self.body)
            else:
                response = self.update_elastic_pool(self.body)

        elif self.to_do == Actions.Delete:
            self.log("SQL Elastic Pool instance deleted")
            self.results['changed'] = True

            if self.check_mode:
                return self.results

            self.delete_elastic_pool()
            # make sure instance is actually deleted, for some Azure resources, instance is hanging around
            # for some time after deletion -- this should be really fixed in Azure
            while self.get_elastic_pool():
                time.sleep(20)
        else:
            self.log("SQL Elastic Pool instance unchanged")
            self.results['changed'] = False
            response = old_response

        self.results['elastic_pool'] = response

        return self.results

    def update_elastic_pool(self, parameters):
        '''
        Creates or updates SQL Elastic Pool with the specified configuration.

        :return: deserialized SQL Elastic Pool instance state dictionary
        '''
        self.log(
            "Creating / Updating the SQL Elastic Pool instance {0}".format(self.name))

        try:
            response = self.sql_client.elastic_pools.begin_update(resource_group_name=self.resource_group,
                                                                  server_name=self.server_name,
                                                                  elastic_pool_name=self.name,
                                                                  parameters=parameters)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
        except Exception as exc:
            self.log('Error attempting to create the SQL Elastic Pool instance.')
            self.fail(
                "Error creating the SQL Elastic Pool instance: {0}".format(str(exc)))
        return self.format_item(response)

    def create_elastic_pool(self, parameters):
        '''
        Creates or updates SQL Elastic Pool with the specified configuration.

        :return: deserialized SQL Elastic Pool instance state dictionary
        '''
        self.log(
            "Creating / Updating the SQL Elastic Pool instance {0}".format(self.name))

        try:
            response = self.sql_client.elastic_pools.begin_create_or_update(resource_group_name=self.resource_group,
                                                                            server_name=self.server_name,
                                                                            elastic_pool_name=self.name,
                                                                            parameters=parameters)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
        except Exception as exc:
            self.log('Error attempting to create the SQL Elastic Pool instance.')
            self.fail(
                "Error creating the SQL Elastic Pool instance: {0}".format(str(exc)))
        return self.format_item(response)

    def delete_elastic_pool(self):
        '''
        Deletes specified SQL Elastic Pool instance in the specified subscription and resource group.

        :return: True
        '''
        self.log("Deleting the SQL Elastic Pool instance {0}".format(self.name))
        try:
            response = self.sql_client.elastic_pools.begin_delete(resource_group_name=self.resource_group,
                                                                  server_name=self.server_name,
                                                                  elastic_pool_name=self.name)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
        except Exception as e:
            self.log('Error attempting to delete the SQL Elastic Pool instance.')
            self.fail(
                "Error deleting the SQL Elastic Pool instance: {0}".format(str(e)))

        return True

    def get_elastic_pool(self):
        '''
        Gets the properties of the specified SQL Elastic Pool.

        :return: deserialized SQL Elastic Pool instance state dictionary
        '''
        found = False
        try:
            response = self.sql_client.elastic_pools.get(resource_group_name=self.resource_group,
                                                         server_name=self.server_name,
                                                         elastic_pool_name=self.name)
            found = True
        except ResourceNotFoundError:
            self.log('Did not find the SQL Elastic Pool instance.')
        if found is True:
            return self.format_item(response)

        return False

    def format_item(self, item):
        if not item:
            return None

        d = dict(
            resource_group=self.resource_group,
            id=item.id,
            name=item.name,
            location=item.location,
            tags=item.tags,
            max_size_bytes=item.max_size_bytes,
            zone_redundant=item.zone_redundant,
            license_type=item.license_type,
            maintenance_configuration_id=item.maintenance_configuration_id,
            per_database_settings=dict(),
            sku=dict()
        )

        if item.sku is not None:
            d['sku']['name'] = item.sku.name
            d['sku']['tier'] = item.sku.tier
            d['sku']['size'] = item.sku.size
            d['sku']['family'] = item.sku.family
            d['sku']['capacity'] = item.sku.capacity
        if item.per_database_settings is not None:
            d['per_database_settings']['min_capacity'] = item.per_database_settings.min_capacity
            d['per_database_settings']['max_capacity'] = item.per_database_settings.max_capacity

        return d


def main():
    """Main execution"""
    AzureRMSqlElasticPool()


if __name__ == '__main__':
    main()



# Copyright (c) 2021 Red Hat
#
# This code is part of Ansible, but is an independent component.
# This particular file snippet, and this file snippet only, is BSD licensed.
# Modules you write using this snippet, which is embedded dynamically by Ansible
# still belong to the author of the module, and may assign their own license
# to the complete work.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright notice,
#      this list of conditions and the following disclaimer in the documentation
#      and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import os

from ansible.plugins.lookup import LookupBase
import ansible_collections.cloud.common.plugins.module_utils.turbo.common
from ansible_collections.cloud.common.plugins.module_utils.turbo.exceptions import (
    EmbeddedModuleUnexpectedFailure,
)


def get_server_ttl(variables):
    # trying to retrieve first TTL from environment variable
    ttl = os.environ.get("ANSIBLE_TURBO_LOOKUP_TTL", None)
    if ttl is not None:
        return ttl
    # Read TTL from ansible environment
    for env_var in variables.get("environment", []):
        value = env_var.get("ANSIBLE_TURBO_LOOKUP_TTL", None)
        test_var_int = [
            isinstance(value, str) and value.isnumeric(),
            isinstance(value, int),
        ]
        if value is not None and any(test_var_int):
            ttl = value
    return ttl


class TurboLookupBase(LookupBase):
    def run_on_daemon(self, terms, variables=None, **kwargs):
        self._ttl = get_server_ttl(variables)
        return self.execute(terms=terms, variables=variables, **kwargs)

    @property
    def socket_path(self):
        if not hasattr(self, "__socket_path"):
            """
            Input:
                _load_name: ansible_collections.cloud.common.plugins.lookup.turbo_random_lookup
            Output:
                __socket_path: {HOME}/.ansible/tmp/turbo_mode_cloud.common.socket
            this will allow to have one socket per collection
            """
            name = self._load_name
            ansible_collections = "ansible_collections."
            if name.startswith(ansible_collections):
                name = name.replace(ansible_collections, "", 1)
                lookup_plugins = ".plugins.lookup."
                idx = name.find(lookup_plugins)
                if idx != -1:
                    name = name[:idx]

            self.__socket_path = os.environ[
                "HOME"
            ] + "/.ansible/tmp/turbo_lookup.{0}.socket".format(name)
        return self.__socket_path

    def execute(self, terms, variables=None, **kwargs):
        result = None
        with ansible_collections.cloud.common.plugins.module_utils.turbo.common.connect(
            socket_path=self.socket_path, ttl=self._ttl, plugin="lookup"
        ) as turbo_socket:
            content = (self._load_name, terms, variables, kwargs)
            (result, errors) = turbo_socket.communicate(content)
            if errors:
                raise EmbeddedModuleUnexpectedFailure(errors)

            return result



# -*- coding: utf-8 -*-
# Copyright (c) 2021, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = '''
  name: groupby_as_dict
  short_description: Transform a sequence of dictionaries to a dictionary where the dictionaries are indexed by an attribute
  version_added: 3.1.0
  author: Felix Fontein (@felixfontein)
  description:
    - Transform a sequence of dictionaries to a dictionary where the dictionaries are indexed by an attribute.
  positional: attribute
  options:
    _input:
      description: A list of dictionaries
      type: list
      elements: dictionary
      required: true
    attribute:
      description: The attribute to use as the key.
      type: str
      required: true
'''

EXAMPLES = '''
- name: Arrange a list of dictionaries as a dictionary of dictionaries
  ansible.builtin.debug:
    msg: "{{ sequence | community.general.groupby_as_dict('key') }}"
  vars:
    sequence:
      - key: value
        foo: bar
      - key: other_value
        baz: bar
  # Produces the following nested structure:
  #
  #  value:
  #    key: value
  #    foo: bar
  #  other_value:
  #    key: other_value
  #    baz: bar
'''

RETURN = '''
  _value:
    description: A dictionary containing the dictionaries from the list as values.
    type: dictionary
'''

from ansible.errors import AnsibleFilterError
from ansible.module_utils.common._collections_compat import Mapping, Sequence


def groupby_as_dict(sequence, attribute):
    '''
    Given a sequence of dictionaries and an attribute name, returns a dictionary mapping
    the value of this attribute to the dictionary.

    If multiple dictionaries in the sequence have the same value for this attribute,
    the filter will fail.
    '''
    if not isinstance(sequence, Sequence):
        raise AnsibleFilterError('Input is not a sequence')

    result = dict()
    for list_index, element in enumerate(sequence):
        if not isinstance(element, Mapping):
            raise AnsibleFilterError('Sequence element #{0} is not a mapping'.format(list_index))
        if attribute not in element:
            raise AnsibleFilterError('Attribute not contained in element #{0} of sequence'.format(list_index))
        result_index = element[attribute]
        if result_index in result:
            raise AnsibleFilterError('Multiple sequence entries have attribute value {0!r}'.format(result_index))
        result[result_index] = element
    return result


class FilterModule(object):
    ''' Ansible list filters '''

    def filters(self):
        return {
            'groupby_as_dict': groupby_as_dict,
        }



# -*- coding: utf-8 -*-

# Copyright (c) 2023 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
name: aws_mq
version_added: 6.1.0
short_description: MQ broker inventory source
description:
  - Get brokers from Amazon Web Services MQ.
  - Uses a YAML configuration file that ends with aws_mq.(yml|yaml).
options:
  regions:
    description:
      - A list of regions in which to describe MQ brokers. Available regions are listed here
        U(https://aws.amazon.com/about-aws/global-infrastructure/regional-product-services/)
    type: list
    elements: str
    default: []
  strict_permissions:
    description: By default if an AccessDenied exception is encountered this plugin will fail. You can set I(strict_permissions) to
      C(False) in the inventory config file which will allow the restrictions to be gracefully skipped.
    type: bool
    default: True
  statuses:
    description:
      - A list of desired states for brokers to be added to inventory. Set to C(['all']) as a shorthand to find everything.
        Possible value are listed here U(https://docs.aws.amazon.com/amazon-mq/latest/developer-guide/broker-statuses.html)
    type: list
    elements: str
    default:
      - RUNNING
      - CREATION_IN_PROGRESS
  hostvars_prefix:
    description:
      - The prefix for host variables names coming from AWS.
    type: str
  hostvars_suffix:
    description:
      - The suffix for host variables names coming from AWS.
    type: str
extends_documentation_fragment:
  - inventory_cache
  - constructed
  - amazon.aws.boto3
  - amazon.aws.common.plugins
  - amazon.aws.region.plugins
  - amazon.aws.assume_role.plugins
author:
  - Ali AlKhalidi (@doteast)
"""

EXAMPLES = r"""
---
# Minimal example using AWS credentials from environment vars or instance role credentials
# Get all brokers in us-east-1 region
plugin: community.aws.aws_mq
regions:
  - ca-central-1

---

# Example multiple regions, ignoring permission errors, and only brokers with state RUNNING
plugin: community.aws.aws_mq
regions:
  - us-east-1
  - us-east-2
strict_permissions: false
statuses:
  - RUNNING

---

# Example group by engine, hostvars custom prefix-suffix, and compose variable from tags
plugin: community.aws.aws_mq
regions:
  - ca-central-1
keyed_groups:
  - key: engine_type
    prefix: mq
compose:
  app: 'tags.Applications|split(",")'
hostvars_prefix: aws_
hostvars_suffix: _mq
"""

try:
    import botocore
except ImportError:
    pass  # will be captured by imported HAS_BOTO3

from ansible.errors import AnsibleError
from ansible.module_utils._text import to_native
from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.plugin_utils.inventory import AWSInventoryBase

broker_attr = [
    "MaintenanceWindowStartTime",
    "AutoMinorVersionUpgrade",
    "AuthenticationStrategy",
    "PubliclyAccessible",
    "EncryptionOptions",
    "HostInstanceType",
    "BrokerInstances",
    "SecurityGroups",
    "DeploymentMode",
    "EngineVersion",
    "StorageType",
    "BrokerState",
    "EngineType",
    "SubnetIds",
    "BrokerArn",
    "BrokerId",
    "Created",
    "Logs",
]

inventory_group = "aws_mq"


def _find_hosts_matching_statuses(hosts, statuses):
    if not statuses:
        statuses = ["RUNNING", "CREATION_IN_PROGRESS"]
    if "all" in statuses:
        return hosts
    valid_hosts = []
    for host in hosts:
        if host.get("BrokerState") in statuses:
            valid_hosts.append(host)
    return valid_hosts


def _get_mq_hostname(host):
    if host.get("BrokerName"):
        return host["BrokerName"]


def _get_broker_host_tags(detail):
    tags = []
    if "Tags" in detail:
        for key, value in detail["Tags"].items():
            tags.append({"Key": key, "Value": value})
    return tags


def _add_details_to_hosts(connection, hosts, strict):
    for host in hosts:
        detail = None
        resource_id = host["BrokerId"]
        try:
            detail = connection.describe_broker(BrokerId=resource_id)
        except is_boto3_error_code("AccessDenied") as e:
            if not strict:
                pass
            else:
                raise AnsibleError(f"Failed to query MQ: {to_native(e)}")
        except (
            botocore.exceptions.BotoCoreError,
            botocore.exceptions.ClientError,
        ) as e:  # pylint: disable=duplicate-except
            raise AnsibleError(f"Failed to query MQ: {to_native(e)}")

        if detail:
            # special handling of tags
            host["Tags"] = _get_broker_host_tags(detail)

            # collect rest of attributes
            for attr in broker_attr:
                if attr in detail:
                    host[attr] = detail[attr]


class InventoryModule(AWSInventoryBase):
    NAME = "community.aws.aws_mq"
    INVENTORY_FILE_SUFFIXES = ("aws_mq.yml", "aws_mq.yaml")

    def __init__(self):
        super(InventoryModule, self).__init__()

    def _get_broker_hosts(self, connection, strict):
        def _boto3_paginate_wrapper(func, *args, **kwargs):
            results = []
            try:
                results = func(*args, **kwargs)
                results = results["BrokerSummaries"]
                _add_details_to_hosts(connection, results, strict)
            except is_boto3_error_code("AccessDenied") as e:  # pylint: disable=duplicate-except
                if not strict:
                    results = []
                else:
                    raise AnsibleError(f"Failed to query MQ: {to_native(e)}")
            except (
                botocore.exceptions.ClientError,
                botocore.exceptions.BotoCoreError,
            ) as e:  # pylint: disable=duplicate-except
                raise AnsibleError(f"Failed to query MQ: {to_native(e)}")
            return results

        return _boto3_paginate_wrapper

    def _get_all_hosts(self, regions, strict, statuses):
        """
        :param regions: a list of regions in which to describe hosts
        :param strict: a boolean determining whether to fail or ignore 403 error codes
        :param statuses: a list of statuses that the returned hosts should match
        :return A list of host dictionaries
        """
        all_instances = []

        for connection, _region in self.all_clients("mq"):
            paginator = connection.get_paginator("list_brokers")
            all_instances.extend(self._get_broker_hosts(connection, strict)(paginator.paginate().build_full_result))
        sorted_hosts = list(sorted(all_instances, key=lambda x: x["BrokerName"]))
        return _find_hosts_matching_statuses(sorted_hosts, statuses)

    def _populate_from_cache(self, cache_data):
        hostvars = cache_data.pop("_meta", {}).get("hostvars", {})
        for group in cache_data:
            if group == "all":
                continue
            self.inventory.add_group(group)
            hosts = cache_data[group].get("hosts", [])
            for host in hosts:
                self._populate_host_vars([host], hostvars.get(host, {}), group)
            self.inventory.add_child("all", group)

    def _populate(self, hosts):
        group = inventory_group
        self.inventory.add_group(group)
        if hosts:
            self._add_hosts(hosts=hosts, group=group)
            self.inventory.add_child("all", group)

    def _format_inventory(self, hosts):
        results = {"_meta": {"hostvars": {}}}
        group = inventory_group
        results[group] = {"hosts": []}
        for host in hosts:
            hostname = _get_mq_hostname(host)
            results[group]["hosts"].append(hostname)
            h = self.inventory.get_host(hostname)
            results["_meta"]["hostvars"][h.name] = h.vars
        return results

    def _add_hosts(self, hosts, group):
        """
        :param hosts: a list of hosts to add to the group
        :param group: name of the group the host list belongs to
        """
        for host in hosts:
            hostname = _get_mq_hostname(host)
            host = camel_dict_to_snake_dict(host, ignore_list=["Tags", "EngineType"])
            host["tags"] = boto3_tag_list_to_ansible_dict(host.get("tags", []))
            if host.get("engine_type"):
                # align value with API spec of all upper
                host["engine_type"] = host.get("engine_type", "").upper()

            self.inventory.add_host(hostname, group=group)
            new_vars = dict()
            hostvars_prefix = self.get_option("hostvars_prefix")
            hostvars_suffix = self.get_option("hostvars_suffix")
            for hostvar, hostval in host.items():
                if hostvars_prefix:
                    hostvar = hostvars_prefix + hostvar
                if hostvars_suffix:
                    hostvar = hostvar + hostvars_suffix
                new_vars[hostvar] = hostval
                self.inventory.set_variable(hostname, hostvar, hostval)
            host.update(new_vars)

            strict = self.get_option("strict")
            self._set_composite_vars(self.get_option("compose"), host, hostname, strict=strict)
            self._add_host_to_composed_groups(self.get_option("groups"), host, hostname, strict=strict)
            self._add_host_to_keyed_groups(self.get_option("keyed_groups"), host, hostname, strict=strict)

    def parse(self, inventory, loader, path, cache=True):
        super().parse(inventory, loader, path, cache=cache)

        # get user specifications
        regions = self.get_option("regions")
        strict_permissions = self.get_option("strict_permissions")
        statuses = self.get_option("statuses")

        result_was_cached, results = self.get_cached_result(path, cache)
        if result_was_cached:
            self._populate_from_cache(results)
            return

        results = self._get_all_hosts(regions, strict_permissions, statuses)
        self._populate(results)

        formatted_inventory = self._format_inventory(results)
        self.update_cached_result(path, cache, formatted_inventory)



#
# This code is part of Ansible, but is an independent component.
#
# This particular file snippet, and this file snippet only, is BSD licensed.
# Modules you write using this snippet, which is embedded dynamically by Ansible
# still belong to the author of the module, and may assign their own license
# to the complete work.
#
# (c) 2017 Red Hat, Inc.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright notice,
#      this list of conditions and the following disclaimer in the documentation
#      and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
from __future__ import absolute_import, division, print_function


__metaclass__ = type
import json
import os
import time

from ansible.module_utils._text import to_text
from ansible.module_utils.connection import Connection, ConnectionError
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.config import (
    NetworkConfig,
    dumps,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    ComplexList,
    to_list,
)


_DEVICE_CONNECTION = None


def get_connection(module):
    global _DEVICE_CONNECTION
    if not _DEVICE_CONNECTION:
        connection_proxy = Connection(module._socket_path)
        cap = json.loads(connection_proxy.get_capabilities())
        if cap["network_api"] == "cliconf":
            conn = Cli(module)
        elif cap["network_api"] == "eapi":
            conn = HttpApi(module)
        _DEVICE_CONNECTION = conn
    return _DEVICE_CONNECTION


def transform_commands(module):
    transform = ComplexList(
        dict(
            command=dict(key=True),
            output=dict(),
            prompt=dict(type="list"),
            answer=dict(type="list"),
            newline=dict(type="bool", default=True),
            sendonly=dict(type="bool", default=False),
            check_all=dict(type="bool", default=False),
            version=dict(
                type="str",
                default="latest",
                choices=["latest", "1"],
            ),
        ),
        module,
    )

    return transform(module.params["commands"])


def session_name():
    """Generate a unique string to be used as a configuration session name."""
    return "ansible_%d" % (time.time() * 100)


class Cli:
    def __init__(self, module):
        self._module = module
        self._device_configs = {}
        self._session_support = None
        self._connection = None

    @property
    def supports_sessions(self):
        if self._session_support is None:
            self._session_support = self._get_connection().supports_sessions()
        return self._session_support

    def _get_connection(self):
        if self._connection:
            return self._connection
        self._connection = Connection(self._module._socket_path)

        return self._connection

    def get_config(self, flags=None):
        """Retrieves the current config from the device or cache"""
        flags = [] if flags is None else flags

        cmd = "show running-config "
        cmd += " ".join(flags)
        cmd = cmd.strip()

        try:
            return self._device_configs[cmd]
        except KeyError:
            conn = self._get_connection()
            try:
                out = conn.get_config(flags=flags)
            except ConnectionError as exc:
                self._module.fail_json(
                    msg=to_text(exc, errors="surrogate_then_replace"),
                )

            cfg = to_text(out, errors="surrogate_then_replace").strip()
            self._device_configs[cmd] = cfg
            return cfg

    def run_commands(self, commands, check_rc=True):
        """Run list of commands on remote device and return results"""
        connection = self._get_connection()
        try:
            response = connection.run_commands(
                commands=commands,
                check_rc=check_rc,
            )
        except ConnectionError as exc:
            self._module.fail_json(
                msg=to_text(exc, errors="surrogate_then_replace"),
            )
        return response

    def get_session_config(self, commands, commit=False, replace=False):
        """Loads the config commands onto the remote device"""
        conn = self._get_connection()
        try:
            response = conn.get_session_config(commands, commit, replace)
        except ConnectionError as exc:
            message = getattr(exc, "err", to_text(exc))
            if "check mode is not supported without configuration session" in message:
                self._module.warn(
                    "EOS can not check config without config session",
                )
                response = {"changed": True}
            else:
                self._module.fail_json(
                    msg="%s" % message,
                    data=to_text(message, errors="surrogate_then_replace"),
                )

        return response

    def load_config(self, commands, commit=False, replace=False):
        """Loads the config commands onto the remote device"""
        conn = self._get_connection()
        try:
            response = conn.edit_config(commands, commit, replace)
        except ConnectionError as exc:
            message = getattr(exc, "err", to_text(exc))
            if "check mode is not supported without configuration session" in message:
                self._module.warn(
                    "EOS can not check config without config session",
                )
                response = {"changed": True}
            else:
                self._module.fail_json(
                    msg="%s" % message,
                    data=to_text(message, errors="surrogate_then_replace"),
                )

        return response

    def get_diff(
        self,
        candidate=None,
        running=None,
        diff_match="line",
        diff_ignore_lines=None,
        path=None,
        diff_replace="line",
    ):
        conn = self._get_connection()
        try:
            diff = conn.get_diff(
                candidate=candidate,
                running=running,
                diff_match=diff_match,
                diff_ignore_lines=diff_ignore_lines,
                path=path,
                diff_replace=diff_replace,
            )
        except ConnectionError as exc:
            self._module.fail_json(
                msg=to_text(exc, errors="surrogate_then_replace"),
            )
        return diff

    def get_capabilities(self):
        """Returns platform info of the remove device"""
        if hasattr(self._module, "_capabilities"):
            return self._module._capabilities

        connection = self._get_connection()
        try:
            capabilities = connection.get_capabilities()
        except ConnectionError as exc:
            self._module.fail_json(
                msg=to_text(exc, errors="surrogate_then_replace"),
            )
        self._module._capabilities = json.loads(capabilities)
        return self._module._capabilities


class HttpApi:
    def __init__(self, module):
        self._module = module
        self._device_configs = {}
        self._session_support = None
        self._connection_obj = None

    @property
    def _connection(self):
        if not self._connection_obj:
            self._connection_obj = Connection(self._module._socket_path)

        return self._connection_obj

    @property
    def supports_sessions(self):
        if self._session_support is None:
            self._session_support = self._connection.supports_sessions()
        return self._session_support

    def run_commands(self, commands, check_rc=True):
        """Runs list of commands on remote device and returns results"""
        output = None
        queue = list()
        responses = list()

        def run_queue(queue, output, version):
            try:
                response = to_list(
                    self._connection.send_request(
                        queue,
                        output=output,
                        version=version,
                    ),
                )
            except ConnectionError as exc:
                if check_rc:
                    raise
                return to_list(to_text(exc))

            if output == "json":
                response = [json.loads(item) for item in response]
            return response

        for item in to_list(commands):
            cmd_output = "text"
            if isinstance(item, dict):
                command = item["command"]
                if "output" in item:
                    cmd_output = item["output"]
                if "version" in item:
                    version = item["version"]
            else:
                command = item

            # Emulate '| json' from CLI
            if is_json(command):
                command = command.rsplit("|", 1)[0]
                cmd_output = "json"

            if output and output != cmd_output:
                responses.extend(run_queue(queue, output, version))
                queue = list()

            output = cmd_output
            queue.append(command)

        if queue:
            responses.extend(run_queue(queue, output, version))

        return responses

    def get_config(self, flags=None):
        """Retrieves the current config from the device or cache"""
        flags = [] if flags is None else flags

        cmd = "show running-config "
        cmd += " ".join(flags)
        cmd = cmd.strip()

        try:
            return self._device_configs[cmd]
        except KeyError:
            try:
                out = self._connection.send_request(cmd)
            except ConnectionError as exc:
                self._module.fail_json(
                    msg=to_text(exc, errors="surrogate_then_replace"),
                )

            cfg = to_text(out).strip()
            self._device_configs[cmd] = cfg
            return cfg

    def get_diff(
        self,
        candidate=None,
        running=None,
        diff_match="line",
        diff_ignore_lines=None,
        path=None,
        diff_replace="line",
    ):
        diff = {}

        # prepare candidate configuration
        candidate_obj = NetworkConfig(indent=3)
        candidate_obj.load(candidate)

        if running and diff_match != "none" and diff_replace != "config":
            # running configuration
            running_obj = NetworkConfig(
                indent=3,
                contents=running,
                ignore_lines=diff_ignore_lines,
            )
            configdiffobjs = candidate_obj.difference(
                running_obj,
                path=path,
                match=diff_match,
                replace=diff_replace,
            )

        else:
            configdiffobjs = candidate_obj.items

        diff["config_diff"] = dumps(configdiffobjs, "commands") if configdiffobjs else {}
        return diff

    def load_config(self, config, commit=False, replace=False):
        """Loads the configuration onto the remote devices

        If the device doesn't support configuration sessions, this will
        fallback to using configure() to load the commands.  If that happens,
        there will be no returned diff or session values
        """
        return self.edit_config(config, commit, replace)

    def edit_config(self, config, commit=False, replace=False):
        """Loads the configuration onto the remote devices

        If the device doesn't support configuration sessions, this will
        fallback to using configure() to load the commands.  If that happens,
        there will be no returned diff or session values
        """
        session = session_name()
        result = {"session": session}
        banner_cmd = None
        banner_input = []

        commands = ["configure session %s" % session]
        if replace:
            commands.append("rollback clean-config")

        for command in config:
            if command.startswith("banner"):
                banner_cmd = command
                banner_input = []
            elif banner_cmd:
                if command == "EOF":
                    command = {
                        "cmd": banner_cmd,
                        "input": "\n".join(banner_input),
                    }
                    banner_cmd = None
                    commands.append(command)
                else:
                    banner_input.append(command)
                    continue
            else:
                commands.append(command)

        try:
            response = self._connection.send_request(commands)
        except Exception:
            commands = ["configure session %s" % session, "abort"]
            response = self._connection.send_request(commands, output="text")
            raise

        commands = [
            "configure session %s" % session,
            "show session-config diffs",
        ]
        if commit:
            commands.append("commit")
        else:
            commands.append("abort")

        response = self._connection.send_request(commands, output="text")
        diff = response[1].strip()
        if diff:
            result["diff"] = diff

        return result

    def get_session_config(self, config, commit=False, replace=False):
        """Loads the configuration onto the remote devices

        If the device doesn't support configuration sessions, this will
        fallback to using configure() to load the commands.  If that happens,
        there will be no returned diff or session values
        """
        resp = ""
        use_session = os.getenv("ANSIBLE_EOS_USE_SESSIONS", True)
        try:
            use_session = int(use_session)
        except ValueError:
            pass

        if not all((bool(use_session), self.supports_sessions)):
            if commit:
                return self.configure(config)
            else:
                self._module.warn(
                    "EOS can not check config without config session",
                )
                result = {"changed": True}
                return result
        session = session_name()
        result = {"session": session}
        commands = ["configure session %s" % session]

        if replace:
            commands.append("rollback clean-config")

        commands.extend(config)
        response = self._connection.send_request(commands)
        if "error" in response:
            commands = ["configure session %s" % session, "abort"]
            self._connection.send_request(commands)
            err = response["error"]
            error_text = []
            for data in err["data"]:
                error_text.extend(data.get("errors", []))
            error_text = "\n".join(error_text) or err["message"]
            self._module.fail_json(msg=error_text, code=err["code"])

        commands = [
            "configure session %s" % session,
            "show session-config",
        ]
        if commit:
            commands.append("commit")
        else:
            commands.append("abort")
        response = self._connection.send_request(commands, output="text")
        for out in response:
            if out:
                resp += out + ""
        return resp.rstrip()

    def get_capabilities(self):
        """Returns platform info of the remove device"""
        try:
            capabilities = self._connection.get_capabilities()
        except ConnectionError as exc:
            self._module.fail_json(
                msg=to_text(exc, errors="surrogate_then_replace"),
            )

        return json.loads(capabilities)


def is_json(cmd):
    return to_text(cmd, errors="surrogate_then_replace").endswith("| json")


def to_command(module, commands):
    transform = ComplexList(
        dict(
            command=dict(key=True),
            output=dict(type="str", default="text"),
            prompt=dict(type="list"),
            answer=dict(type="list"),
            newline=dict(type="bool", default=True),
            sendonly=dict(type="bool", default=False),
            check_all=dict(type="bool", default=False),
            version=dict(type="str", default="latest"),
        ),
        module,
    )

    return transform(to_list(commands))


def get_config(module, flags=None):
    flags = None if flags is None else flags

    conn = get_connection(module)
    return conn.get_config(flags)


def run_commands(module, commands, check_rc=True):
    conn = get_connection(module)
    return conn.run_commands(to_command(module, commands), check_rc=check_rc)


def load_config(module, config, commit=False, replace=False):
    conn = get_connection(module)
    return conn.load_config(config, commit, replace)


def get_session_config(module, config, commit=False, replace=False):
    conn = get_connection(module)
    return conn.get_session_config(config, commit, replace)


def get_diff(
    self,
    candidate=None,
    running=None,
    diff_match="line",
    diff_ignore_lines=None,
    path=None,
    diff_replace="line",
):
    conn = self.get_connection()
    return conn.get_diff(
        candidate=candidate,
        running=running,
        diff_match=diff_match,
        diff_ignore_lines=diff_ignore_lines,
        path=path,
        diff_replace=diff_replace,
    )


def get_capabilities(module):
    conn = get_connection(module)
    return conn.get_capabilities()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: transport_gateway_settings_info
short_description: Information module for Transport Gateway Settings
description:
- Get all Transport Gateway Settings.
- Transport Gateway acts a proxy for the communication between the ISE servers in your network and the Telemetry servers in case of air-gapped network.
version_added: '2.1.0'
extends_documentation_fragment:
  - cisco.ise.module_info
author: Rafael Campos (@racampos)
options: {}
requirements:
- ciscoisesdk >= 2.1.1
- python >= 3.5
seealso:
- name: Cisco ISE documentation for telemetry
  description: Complete reference of the telemetry API.
  link: https://developer.cisco.com/docs/identity-services-engine/v1/#!system-settings-openapi
notes:
  - SDK Method used are
    telemetry.Telemetry.get_transport_gateway,

  - Paths used are
    get /api/v1/system-settings/telemetry/transport-gateway,

"""

EXAMPLES = r"""
- name: Get all Transport Gateway Settings
  cisco.ise.transport_gateway_settings_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
  register: result

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "enableTransportGateway": true,
      "url": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: autoscaling_instance_refresh
version_added: 3.2.0
short_description: Start or cancel an EC2 Auto Scaling Group (ASG) instance refresh in AWS
description:
  - Start or cancel an EC2 Auto Scaling Group instance refresh in AWS.
  - Can be used with M(community.aws.autoscaling_instance_refresh_info) to track the subsequent progress.
  - Prior to release 5.0.0 this module was called C(community.aws.ec2_asg_instance_refresh).
    The usage did not change.
author:
  - "Dan Khersonsky (@danquixote)"
options:
  state:
    description:
      - Desired state of the ASG.
    type: str
    required: true
    choices: [ 'started', 'cancelled' ]
  name:
    description:
      - The name of the auto scaling group you are searching for.
    type: str
    required: true
  strategy:
    description:
      - The strategy to use for the instance refresh. The only valid value is C(Rolling).
      - A rolling update is an update that is applied to all instances in an Auto Scaling group until all instances have been updated.
      - A rolling update can fail due to failed health checks or if instances are on standby or are protected from scale in.
      - If the rolling update process fails, any instances that were already replaced are not rolled back to their previous configuration.
    type: str
    default: 'Rolling'
  preferences:
    description:
      - Set of preferences associated with the instance refresh request.
      - If not provided, the default values are used.
      - For I(min_healthy_percentage), the default value is C(90).
      - For I(instance_warmup), the default is to use the value specified for the health check grace period for the Auto Scaling group.
      - Can not be specified when I(state) is set to 'cancelled'.
    required: false
    suboptions:
      min_healthy_percentage:
        description:
          - Total percent of capacity in ASG that must remain healthy during instance refresh to allow operation to continue.
          - It is rounded up to the nearest integer.
        type: int
        default: 90
      instance_warmup:
        description:
          - The number of seconds until a newly launched instance is configured and ready to use.
          - During this time, Amazon EC2 Auto Scaling does not immediately move on to the next replacement.
          - The default is to use the value for the health check grace period defined for the group.
        type: int
    type: dict
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Start a refresh
  community.aws.autoscaling_instance_refresh:
    name: some-asg
    state: started

- name: Cancel a refresh
  community.aws.autoscaling_instance_refresh:
    name: some-asg
    state: cancelled

- name: Start a refresh and pass preferences
  community.aws.autoscaling_instance_refresh:
    name: some-asg
    state: started
    preferences:
      min_healthy_percentage: 91
      instance_warmup: 60
"""

RETURN = r"""
---
instance_refresh_id:
    description: instance refresh id
    returned: success
    type: str
    sample: "08b91cf7-8fa6-48af-b6a6-d227f40f1b9b"
auto_scaling_group_name:
    description: Name of autoscaling group
    returned: success
    type: str
    sample: "public-webapp-production-1"
status:
    description:
      -  The current state of the group when DeleteAutoScalingGroup is in progress.
      -  The following are the possible statuses
      -    Pending --  The request was created, but the operation has not started.
      -    InProgress --  The operation is in progress.
      -    Successful --  The operation completed successfully.
      -    Failed --  The operation failed to complete. You can troubleshoot using the status reason and the scaling activities.
      -    Cancelling --
      -        An ongoing operation is being cancelled.
      -        Cancellation does not roll back any replacements that have already been completed,
      -        but it prevents new replacements from being started.
      -    Cancelled --  The operation is cancelled.
    returned: success
    type: str
    sample: "Pending"
start_time:
    description: The date and time this ASG was created, in ISO 8601 format.
    returned: success
    type: str
    sample: "2015-11-25T00:05:36.309Z"
end_time:
    description: The date and time this ASG was created, in ISO 8601 format.
    returned: success
    type: str
    sample: "2015-11-25T00:05:36.309Z"
percentage_complete:
    description: the % of completeness
    returned: success
    type: int
    sample: 100
instances_to_update:
    description: num. of instance to update
    returned: success
    type: int
    sample: 5
"""

try:
    from botocore.exceptions import BotoCoreError
    from botocore.exceptions import ClientError
except ImportError:
    pass  # caught by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict
from ansible.module_utils.common.dict_transformations import snake_dict_to_camel_dict

from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.transformation import scrub_none_parameters

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def start_or_cancel_instance_refresh(conn, module):
    """
    Args:
        conn (boto3.AutoScaling.Client): Valid Boto3 ASG client.
        module: AnsibleAWSModule object

    Returns:
        {
            "instance_refreshes": [
                    {
                        'auto_scaling_group_name': 'ansible-test-hermes-63642726-asg',
                        'instance_refresh_id': '6507a3e5-4950-4503-8978-e9f2636efc09',
                        'instances_to_update': 1,
                        'percentage_complete': 0,
                        "preferences": {
                            "instance_warmup": 60,
                            "min_healthy_percentage": 90,
                            "skip_matching": false
                        },
                        'start_time': '2021-02-04T03:39:40+00:00',
                        'status': 'Cancelling',
                        'status_reason': 'Replacing instances before cancelling.',
                    }
              ]
        }
    """

    asg_state = module.params.get("state")
    asg_name = module.params.get("name")
    preferences = module.params.get("preferences")

    args = {}
    args["AutoScalingGroupName"] = asg_name
    if asg_state == "started":
        args["Strategy"] = module.params.get("strategy")
    if preferences:
        if asg_state == "cancelled":
            module.fail_json(msg="can not pass preferences dict when canceling a refresh")
        _prefs = scrub_none_parameters(preferences)
        args["Preferences"] = snake_dict_to_camel_dict(_prefs, capitalize_first=True)
    cmd_invocations = {
        "cancelled": conn.cancel_instance_refresh,
        "started": conn.start_instance_refresh,
    }
    try:
        if module.check_mode:
            if asg_state == "started":
                ongoing_refresh = conn.describe_instance_refreshes(AutoScalingGroupName=asg_name).get(
                    "InstanceRefreshes", "[]"
                )
                if ongoing_refresh:
                    module.exit_json(
                        changed=False,
                        msg="In check_mode - Instance Refresh is already in progress, can not start new instance refresh.",
                    )
                else:
                    module.exit_json(changed=True, msg="Would have started instance refresh if not in check mode.")
            elif asg_state == "cancelled":
                ongoing_refresh = conn.describe_instance_refreshes(AutoScalingGroupName=asg_name).get(
                    "InstanceRefreshes", "[]"
                )[0]
                if ongoing_refresh.get("Status", "") in ["Cancelling", "Cancelled"]:
                    module.exit_json(
                        changed=False,
                        msg="In check_mode - Instance Refresh already cancelled or is pending cancellation.",
                    )
                elif not ongoing_refresh:
                    module.exit_json(chaned=False, msg="In check_mode - No active referesh found, nothing to cancel.")
                else:
                    module.exit_json(changed=True, msg="Would have cancelled instance refresh if not in check mode.")
        result = cmd_invocations[asg_state](aws_retry=True, **args)
        instance_refreshes = conn.describe_instance_refreshes(
            AutoScalingGroupName=asg_name, InstanceRefreshIds=[result["InstanceRefreshId"]]
        )
        result = dict(instance_refreshes=camel_dict_to_snake_dict(instance_refreshes["InstanceRefreshes"][0]))
        return module.exit_json(**result)
    except (BotoCoreError, ClientError) as e:
        module.fail_json_aws(e, msg=f"Failed to {asg_state.replace('ed', '')} InstanceRefresh")


def main():
    argument_spec = dict(
        state=dict(
            type="str",
            required=True,
            choices=["started", "cancelled"],
        ),
        name=dict(required=True),
        strategy=dict(type="str", default="Rolling", required=False),
        preferences=dict(
            type="dict",
            required=False,
            options=dict(
                min_healthy_percentage=dict(type="int", default=90),
                instance_warmup=dict(type="int"),
            ),
        ),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )
    autoscaling = module.client(
        "autoscaling",
        retry_decorator=AWSRetry.jittered_backoff(retries=10, catch_extra_error_codes=["InstanceRefreshInProgress"]),
    )

    start_or_cancel_instance_refresh(autoscaling, module)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2021 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


class ModuleDocFragment(object):

    # Standard files documentation fragment
    DOCUMENTATION = r'''
description:
    - The module allows to set, modify and delete multiple DNS record sets at once.
    - With the O(prune) option, it is also possible to delete existing record sets
      that are not mentioned in the module parameters. With this, it is possible
      to synchronize the expected state of a DNS zone with the expected state.
    - "It is possible to ignore certain record sets by specifying O(record_sets[].ignore=true) for
       that record set."

options:
    zone_name:
        description:
          - The DNS zone to modify.
          - Exactly one of O(zone_name) and O(zone_id) must be specified.
        type: str
        aliases:
          - zone
    zone_id:
        description:
          - The ID of the DNS zone to modify.
          - Exactly one of O(zone_name) and O(zone_id) must be specified.
    prune:
        description:
          - If set to V(true), will remove all existing records in the zone that are not listed in O(record_sets).
        type: bool
        default: false
    record_sets:
        description:
          - The records that should be present in the zone.
        required: true
        type: list
        elements: dict
        aliases:
          - records
        suboptions:
            # (The following must be kept in sync with the equivalent lines in <provider_name>.py!)
            record:
                description:
                  - The full DNS record to create or delete.
                  - Exactly one of O(record_sets[].record) and O(record_sets[].prefix) must be specified.
                type: str
            prefix:
                description:
                  - The prefix of the DNS record.
                  - This is the part of O(record_sets[].record) before O(zone_name). For example, if the record to be modified is C(www.example.com)
                    for the zone C(example.com), the prefix is V(www). If the record in this example would be C(example.com), the
                    prefix would be V('') (empty string).
                  - Exactly one of O(record_sets[].record) and O(record_sets[].prefix) must be specified.
                type: str
            ttl:
                description:
                  - The TTL to give the new record, in seconds.
                type: int
            type:
                description:
                  - The type of DNS record to create or delete.
                required: true
                type: str
            value:
                description:
                  - The new value when creating a DNS record.
                  - YAML lists or multiple comma-spaced values are allowed.
                  - When deleting a record all values for the record must be specified or it will
                    not be deleted.
                  - Must be specified if O(record_sets[].ignore=false).
                type: list
                elements: str
            ignore:
                description:
                  - If set to V(true), O(record_sets[].value) will be ignored.
                  - This is useful when O(prune=true), but you do not want certain entries to be removed
                    without having to know their current value.
                type: bool
                default: false
'''



#
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The eos_snmp_server config file.
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to its desired end-state is
created.
"""

import re

from ansible.module_utils.six import iteritems
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module import (
    ResourceModule,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_merge,
    get_from_dict,
)

from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.facts import Facts
from ansible_collections.arista.eos.plugins.module_utils.network.eos.rm_templates.snmp_server import (
    Snmp_serverTemplate,
)


class Snmp_server(ResourceModule):
    """
    The eos_snmp_server config class
    """

    def __init__(self, module):
        super(Snmp_server, self).__init__(
            empty_fact_val={},
            facts_module=Facts(module),
            module=module,
            resource="snmp_server",
            tmplt=Snmp_serverTemplate(),
        )
        self.parsers = [
            "chassis_id",
            "contact",
            "traps.bgp",
            "traps.bridge",
            "traps.capacity",
            "traps.entity",
            "traps.external_alarm",
            "traps.isis",
            "traps.lldp",
            "traps.mpls_ldp",
            "traps.msdp",
            "traps.ospf",
            "traps.ospfv3",
            "traps.pim",
            "traps.snmp",
            "traps.snmpConfigManEvent",
            "traps.switchover",
            "traps.test",
            "traps.vrrp",
            "engineid",
            "extension",
            "local_interface",
            "location",
            "notification",
            "objects.mac",
            "objects.route",
            "qos",
            "qosmib",
            "transmit",
            "transport",
        ]

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        if self.state not in ["parsed", "gathered"]:
            self.generate_commands()
            self.run_commands()
        return self.result

    def generate_commands(self):
        """Generate configuration commands to send based on
        want, have and desired state.
        """

        wantd = {"snmp_server": self.want}
        haved = {"snmp_server": self.have}

        # turn all lists of dicts into dicts prior to merge
        for entry in wantd["snmp_server"], haved["snmp_server"]:
            self._snmp_server_list_to_dict(entry)

        # if state is merged, merge want onto have and then compare
        if self.state == "merged":
            wantd = dict_merge(haved, wantd)

        # if state is deleted, empty out wantd and set haved to wantd
        if self.state == "deleted":
            haved = {k: v for k, v in iteritems(haved) if k in wantd or not wantd}
            wantd = {}

        # if state is deleted, empty out wantd and set haved to wantd
        if self.state == "deleted":
            wantd = {}
            for k, have in iteritems(haved):
                self._compare(want={}, have=have)

        for k, want in iteritems(wantd):
            self._compare(want=want, have=haved.pop(k, {}))

    def _compare(self, want, have):
        """Leverages the base class `compare()` method and
        populates the list of commands to be run by comparing
        the `want` and `have` data with the `parsers` defined
        for the Snmp_server network resource.
        """
        self._compare_hosts(want, have)
        self._compare_lists(want, have)
        for name, entry in iteritems(want):
            self.compare(
                parsers=self.parsers,
                want={name: entry},
                have={name: have.pop(name, {})},
            )
        for name, entry in iteritems(have):
            self.compare(parsers=self.parsers, want={}, have={name: entry})

        self._modify_traps_negate()

    def _modify_traps_negate(self):
        command_set = []
        for cmd in self.commands:
            if re.search("no snmp-server enable traps", cmd):
                command_set.append(cmd.replace("no ", "default "))
            else:
                command_set.append(cmd)
        self.commands = command_set

    def _compare_lists(self, want, have):
        parsers = [
            "communities_ipv6_acl",
            "communities_ipv4_acl",
            "groups",
            "acls",
            "views",
            "users.auth",
            "users.localized",
            "vrfs",
        ]
        for attrib in [
            "communities",
            "groups",
            "acls",
            "users",
            "views",
            "vrfs",
        ]:
            wdict = get_from_dict(want, attrib) or {}
            hdict = get_from_dict(have, attrib) or {}
            for key, entry in iteritems(wdict):
                # self.addcmd(entry, attrib, False)
                self.compare(
                    parsers=parsers,
                    want={attrib: entry},
                    have={attrib: hdict.pop(key, {})},
                )
            # remove remaining items in have for replaced
            for entry in hdict.values():
                self.compare(parsers=parsers, want={}, have={attrib: entry})

    def _compare_hosts(self, want, have):
        wdict = get_from_dict(want, "hosts") or {}
        hdict = get_from_dict(have, "hosts") or {}
        for key, entry in iteritems(wdict):
            # self.addcmd(entry, attrib, False)
            self.compare(
                parsers="hosts",
                want={"hosts": {key: entry}},
                have={"hosts": {key: hdict.pop(key, {})}},
            )
        # remove remaining items in have for replaced
        for key, entry in iteritems(hdict):
            self.compare(
                parsers="hosts",
                want={},
                have={"hosts": {key: entry}},
            )

    def _snmp_server_list_to_dict(self, entry):
        param_dict = {
            "communities": "name",
            "groups": "group",
            "acls": "afi",
            "users": "user",
            "views": "view",
            "vrfs": "vrf",
        }
        for k, v in iteritems(param_dict):
            if k in entry:
                a_dict = {}
                for el in entry[k]:
                    a_dict.update({el[v]: el})
                entry[k] = a_dict
        if "hosts" in entry:
            host_dict = {}
            for el in entry["hosts"]:
                tr = ""
                inf = ""
                if el.get("traps"):
                    tr = "traps"
                if el.get("informs"):
                    inf = "informs"
                host_dict.update(
                    {
                        (
                            el.get("host"),
                            el.get("user"),
                            el.get("version"),
                            inf,
                            tr,
                            el.get("udp_port"),
                        ): el,
                    },
                )
            entry["hosts"] = host_dict



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Rob White (@wimnat)
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: glue_job
version_added: 1.0.0
short_description: Manage an AWS Glue job
description:
  - Manage an AWS Glue job. See U(https://aws.amazon.com/glue/) for details.
  - Prior to release 5.0.0 this module was called C(community.aws.aws_glue_job).
    The usage did not change.
author:
  - "Rob White (@wimnat)"
  - "Vijayanand Sharma (@vijayanandsharma)"
options:
  allocated_capacity:
    description:
      - The number of AWS Glue data processing units (DPUs) to allocate to this Job. From 2 to 100 DPUs
        can be allocated; the default is 10. A DPU is a relative measure of processing power that consists
        of 4 vCPUs of compute capacity and 16 GB of memory.
    type: int
  command_name:
    description:
      - The name of the job command. This must be 'glueetl'.
    default: glueetl
    type: str
  command_python_version:
    description:
      - Python version being used to execute a Python shell job.
      - AWS currently supports C('2') or C('3').
    type: str
    version_added: 2.2.0
  command_script_location:
    description:
      - The S3 path to a script that executes a job.
      - Required when I(state=present).
    type: str
  connections:
    description:
      - A list of Glue connections used for this job.
    type: list
    elements: str
  default_arguments:
    description:
      - A dict of default arguments for this job.  You can specify arguments here that your own job-execution
        script consumes, as well as arguments that AWS Glue itself consumes.
    type: dict
  description:
    description:
      - Description of the job being defined.
    type: str
  glue_version:
    description:
      - Glue version determines the versions of Apache Spark and Python that AWS Glue supports.
    type: str
    version_added: 1.5.0
  max_concurrent_runs:
    description:
      - The maximum number of concurrent runs allowed for the job. The default is 1. An error is returned when
        this threshold is reached. The maximum value you can specify is controlled by a service limit.
    type: int
  max_retries:
    description:
      -  The maximum number of times to retry this job if it fails.
    type: int
  name:
    description:
      - The name you assign to this job definition. It must be unique in your account.
    required: true
    type: str
  number_of_workers:
    description:
      - The number of workers of a defined workerType that are allocated when a job runs.
    type: int
    version_added: 1.5.0
  role:
    description:
      - The name or ARN of the IAM role associated with this job.
      - Required when I(state=present).
    type: str
  state:
    description:
      - Create or delete the AWS Glue job.
    required: true
    choices: [ 'present', 'absent' ]
    type: str
  timeout:
    description:
      - The job timeout in minutes.
    type: int
  worker_type:
    description:
      - The type of predefined worker that is allocated when a job runs.
    choices: [ 'Standard', 'G.1X', 'G.2X' ]
    type: str
    version_added: 1.5.0
notes:
  - Support for I(tags) and I(purge_tags) was added in release 2.2.0.
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.tags
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

# Create an AWS Glue job
- community.aws.glue_job:
    command_script_location: "s3://s3bucket/script.py"
    default_arguments:
      "--extra-py-files": s3://s3bucket/script-package.zip
      "--TempDir": "s3://s3bucket/temp/"
    name: my-glue-job
    role: my-iam-role
    state: present

# Delete an AWS Glue job
- community.aws.glue_job:
    name: my-glue-job
    state: absent
"""

RETURN = r"""
allocated_capacity:
    description: The number of AWS Glue data processing units (DPUs) allocated to runs of this job. From 2 to
                 100 DPUs can be allocated; the default is 10. A DPU is a relative measure of processing power
                 that consists of 4 vCPUs of compute capacity and 16 GB of memory.
    returned: when state is present
    type: int
    sample: 10
command:
    description: The JobCommand that executes this job.
    returned: when state is present
    type: complex
    contains:
        name:
            description: The name of the job command.
            returned: when state is present
            type: str
            sample: glueetl
        script_location:
            description: Specifies the S3 path to a script that executes a job.
            returned: when state is present
            type: str
            sample: mybucket/myscript.py
        python_version:
            description: Specifies the Python version.
            returned: when state is present
            type: str
            sample: 3
connections:
    description: The connections used for this job.
    returned: when state is present
    type: dict
    sample: "{ Connections: [ 'list', 'of', 'connections' ] }"
created_on:
    description: The time and date that this job definition was created.
    returned: when state is present
    type: str
    sample: "2018-04-21T05:19:58.326000+00:00"
default_arguments:
    description: The default arguments for this job, specified as name-value pairs.
    returned: when state is present
    type: dict
    sample: "{ 'mykey1': 'myvalue1' }"
description:
    description: Description of the job being defined.
    returned: when state is present
    type: str
    sample: My first Glue job
glue_version:
    description: Glue version.
    returned: when state is present
    type: str
    sample: 2.0
job_name:
    description: The name of the AWS Glue job.
    returned: always
    type: str
    sample: my-glue-job
execution_property:
    description: An ExecutionProperty specifying the maximum number of concurrent runs allowed for this job.
    returned: always
    type: complex
    contains:
        max_concurrent_runs:
            description: The maximum number of concurrent runs allowed for the job. The default is 1. An error is
                         returned when this threshold is reached. The maximum value you can specify is controlled by
                         a service limit.
            returned: when state is present
            type: int
            sample: 1
last_modified_on:
    description: The last point in time when this job definition was modified.
    returned: when state is present
    type: str
    sample: "2018-04-21T05:19:58.326000+00:00"
max_retries:
    description: The maximum number of times to retry this job after a JobRun fails.
    returned: when state is present
    type: int
    sample: 5
name:
    description: The name assigned to this job definition.
    returned: when state is present
    type: str
    sample: my-glue-job
role:
    description: The name or ARN of the IAM role associated with this job.
    returned: when state is present
    type: str
    sample: my-iam-role
timeout:
    description: The job timeout in minutes.
    returned: when state is present
    type: int
    sample: 300
"""

import copy

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.iam import get_aws_account_info
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.tagging import compare_aws_tags

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def _get_glue_job(connection, module, glue_job_name):
    """
    Get an AWS Glue job based on name. If not found, return None.

    :param connection: AWS boto3 glue connection
    :param module: Ansible module
    :param glue_job_name: Name of Glue job to get
    :return: boto3 Glue job dict or None if not found
    """
    try:
        return connection.get_job(aws_retry=True, JobName=glue_job_name)["Job"]
    except is_boto3_error_code("EntityNotFoundException"):
        return None
    except (
        botocore.exceptions.ClientError,
        botocore.exceptions.BotoCoreError,
    ) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e)


def _compare_glue_job_params(user_params, current_params):
    """
    Compare Glue job params. If there is a difference, return True immediately else return False

    :param user_params: the Glue job parameters passed by the user
    :param current_params: the Glue job parameters currently configured
    :return: True if any parameter is mismatched else False
    """
    # Weirdly, boto3 doesn't return some keys if the value is empty e.g. Description
    # To counter this, add the key if it's missing with a blank value

    if "Description" not in current_params:
        current_params["Description"] = ""
    if "DefaultArguments" not in current_params:
        current_params["DefaultArguments"] = dict()

    if "AllocatedCapacity" in user_params and user_params["AllocatedCapacity"] != current_params["AllocatedCapacity"]:
        return True
    if "Command" in user_params:
        if user_params["Command"]["ScriptLocation"] != current_params["Command"]["ScriptLocation"]:
            return True
        if user_params["Command"]["PythonVersion"] != current_params["Command"]["PythonVersion"]:
            return True
    if "Connections" in user_params and user_params["Connections"] != current_params["Connections"]:
        return True
    if "DefaultArguments" in user_params and user_params["DefaultArguments"] != current_params["DefaultArguments"]:
        return True
    if "Description" in user_params and user_params["Description"] != current_params["Description"]:
        return True
    if (
        "ExecutionProperty" in user_params
        and user_params["ExecutionProperty"]["MaxConcurrentRuns"]
        != current_params["ExecutionProperty"]["MaxConcurrentRuns"]
    ):
        return True
    if "GlueVersion" in user_params and user_params["GlueVersion"] != current_params["GlueVersion"]:
        return True
    if "MaxRetries" in user_params and user_params["MaxRetries"] != current_params["MaxRetries"]:
        return True
    if "Role" in user_params and user_params["Role"] != current_params["Role"]:
        return True
    if "Timeout" in user_params and user_params["Timeout"] != current_params["Timeout"]:
        return True
    if "GlueVersion" in user_params and user_params["GlueVersion"] != current_params["GlueVersion"]:
        return True
    if "WorkerType" in user_params and user_params["WorkerType"] != current_params["WorkerType"]:
        return True
    if "NumberOfWorkers" in user_params and user_params["NumberOfWorkers"] != current_params["NumberOfWorkers"]:
        return True

    return False


def ensure_tags(connection, module, glue_job):
    changed = False

    if module.params.get("tags") is None:
        return False

    account_id, partition = get_aws_account_info(module)
    arn = f"arn:{partition}:glue:{module.region}:{account_id}:job/{module.params.get('name')}"

    try:
        existing_tags = connection.get_tags(aws_retry=True, ResourceArn=arn).get("Tags", {})
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        if module.check_mode:
            existing_tags = {}
        else:
            module.fail_json_aws(e, msg=f"Unable to get tags for Glue job {module.params.get('name')}")

    tags_to_add, tags_to_remove = compare_aws_tags(
        existing_tags, module.params.get("tags"), module.params.get("purge_tags")
    )

    if tags_to_remove:
        changed = True
        if not module.check_mode:
            try:
                connection.untag_resource(aws_retry=True, ResourceArn=arn, TagsToRemove=tags_to_remove)
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(e, msg=f"Unable to set tags for Glue job {module.params.get('name')}")

    if tags_to_add:
        changed = True
        if not module.check_mode:
            try:
                connection.tag_resource(aws_retry=True, ResourceArn=arn, TagsToAdd=tags_to_add)
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(e, msg=f"Unable to set tags for Glue job {module.params.get('name')}")

    return changed


def create_or_update_glue_job(connection, module, glue_job):
    """
    Create or update an AWS Glue job

    :param connection: AWS boto3 glue connection
    :param module: Ansible module
    :param glue_job: a dict of AWS Glue job parameters or None
    :return:
    """

    changed = False
    params = dict()
    params["Name"] = module.params.get("name")
    params["Role"] = module.params.get("role")
    if module.params.get("allocated_capacity") is not None:
        params["AllocatedCapacity"] = module.params.get("allocated_capacity")
    if module.params.get("command_script_location") is not None:
        params["Command"] = {
            "Name": module.params.get("command_name"),
            "ScriptLocation": module.params.get("command_script_location"),
        }
        if module.params.get("command_python_version") is not None:
            params["Command"]["PythonVersion"] = module.params.get("command_python_version")
    if module.params.get("connections") is not None:
        params["Connections"] = {"Connections": module.params.get("connections")}
    if module.params.get("default_arguments") is not None:
        params["DefaultArguments"] = module.params.get("default_arguments")
    if module.params.get("description") is not None:
        params["Description"] = module.params.get("description")
    if module.params.get("glue_version") is not None:
        params["GlueVersion"] = module.params.get("glue_version")
    if module.params.get("max_concurrent_runs") is not None:
        params["ExecutionProperty"] = {"MaxConcurrentRuns": module.params.get("max_concurrent_runs")}
    if module.params.get("max_retries") is not None:
        params["MaxRetries"] = module.params.get("max_retries")
    if module.params.get("timeout") is not None:
        params["Timeout"] = module.params.get("timeout")
    if module.params.get("glue_version") is not None:
        params["GlueVersion"] = module.params.get("glue_version")
    if module.params.get("worker_type") is not None:
        params["WorkerType"] = module.params.get("worker_type")
    if module.params.get("number_of_workers") is not None:
        params["NumberOfWorkers"] = module.params.get("number_of_workers")

    # If glue_job is not None then check if it needs to be modified, else create it
    if glue_job:
        if _compare_glue_job_params(params, glue_job):
            try:
                # Update job needs slightly modified params
                update_params = {"JobName": params["Name"], "JobUpdate": copy.deepcopy(params)}
                del update_params["JobUpdate"]["Name"]
                if not module.check_mode:
                    connection.update_job(aws_retry=True, **update_params)
                changed = True
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(e)
    else:
        try:
            if not module.check_mode:
                connection.create_job(aws_retry=True, **params)
            changed = True
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e)

    glue_job = _get_glue_job(connection, module, params["Name"])

    changed |= ensure_tags(connection, module, glue_job)

    module.exit_json(changed=changed, **camel_dict_to_snake_dict(glue_job or {}, ignore_list=["DefaultArguments"]))


def delete_glue_job(connection, module, glue_job):
    """
    Delete an AWS Glue job

    :param connection: AWS boto3 glue connection
    :param module: Ansible module
    :param glue_job: a dict of AWS Glue job parameters or None
    :return:
    """
    changed = False

    if glue_job:
        try:
            if not module.check_mode:
                connection.delete_job(aws_retry=True, JobName=glue_job["Name"])
            changed = True
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e)

    module.exit_json(changed=changed)


def main():
    argument_spec = dict(
        allocated_capacity=dict(type="int"),
        command_name=dict(type="str", default="glueetl"),
        command_python_version=dict(type="str"),
        command_script_location=dict(type="str"),
        connections=dict(type="list", elements="str"),
        default_arguments=dict(type="dict"),
        description=dict(type="str"),
        glue_version=dict(type="str"),
        max_concurrent_runs=dict(type="int"),
        max_retries=dict(type="int"),
        name=dict(required=True, type="str"),
        number_of_workers=dict(type="int"),
        purge_tags=dict(type="bool", default=True),
        role=dict(type="str"),
        state=dict(required=True, choices=["present", "absent"], type="str"),
        tags=dict(type="dict", aliases=["resource_tags"]),
        timeout=dict(type="int"),
        worker_type=dict(choices=["Standard", "G.1X", "G.2X"], type="str"),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        required_if=[("state", "present", ["role", "command_script_location"])],
        supports_check_mode=True,
    )

    retry_decorator = AWSRetry.jittered_backoff(retries=10)
    connection = module.client("glue", retry_decorator=retry_decorator)

    state = module.params.get("state")

    glue_job = _get_glue_job(connection, module, module.params.get("name"))

    if state == "present":
        create_or_update_glue_job(connection, module, glue_job)
    else:
        delete_glue_job(connection, module, glue_job)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Ospf_interfaces parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


def _tmplt_authentication(data):
    auth = data.get("authentication")
    cmd = "ip ospf authentication"

    if auth.get("enable") is False:
        cmd = "no " + cmd
    else:
        if auth.get("message_digest"):
            cmd += " message-digest"
        elif auth.get("null_auth"):
            cmd += " null"
    return cmd


class Ospf_interfacesTemplate(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(Ospf_interfacesTemplate, self).__init__(lines=lines, tmplt=self, module=module)

    # fmt: off
    PARSERS = [
        {
            "name": "interface",
            "getval": re.compile(
                r'''
              ^interface
              \s(?P<name>\S+)$''', re.VERBOSE,
            ),
            "setval": "interface {{ name }}",
            "result": {
                "{{ name }}": {
                    "name": "{{ name }}",
                    "address_family": {},
                },
            },
            "shared": True,
        },
        {
            "name": "area",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip|ipv6)
                \srouter\s(ospf|ospfv3)
                \s(?P<process_id>\S+)
                \sarea\s(?P<area_id>\S+)
                (\s(?P<secondaries>secondaries\snone))?$""",
                re.VERBOSE,
            ),
            "setval": "{{ 'ip' if afi == 'ipv4' else 'ipv6' }} "
                      "router {{ 'ospf' if afi == 'ipv4' else 'ospfv3' }} "
                      "{{ process_id }} area {{ area.area_id }}{{ ' secondaries none' if area.secondaries|default('True') == False else '' }}",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                        "secondaries": "{{ False if secondaries is defined else None }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "processes_multi_areas",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip|ipv6)
                \srouter\s(ospf|ospfv3)
                \s(?P<process_id>\S+)
                \smulti-area\s(?P<area>\S+)$""",
                re.VERBOSE,
            ),
            "setval": "{{ 'ip' if afi == 'ipv4' else 'ipv6' }} "
                      "router {{ 'ospf' if afi == 'ipv4' else 'ospfv3' }} "
                      "{{ process_id }} multi-area {{ area }}",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "multi_areas": [
                                        "{{ area }}",
                                    ],
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "multi_areas",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip|ipv6)
                \srouter\s(ospf|ospfv3)
                \smulti-area\s(?P<area>\S+)$""",
                re.VERBOSE,
            ),
            "setval": "{{ 'ip' if afi == 'ipv4' else 'ipv6' }} "
                      "router {{ 'ospf' if afi == 'ipv4' else 'ospfv3' }} "
                      "multi-area {{ area }}",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                            "multi_areas": [
                                "{{ area }}",
                            ],
                        },
                    },
                },
            },
        },
        {
            "name": "authentication",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip|ipv6)
                \s(ospf|ospfv3)
                \s(?P<authentication>authentication)
                (\s(?P<opt>(message-digest|null)))?$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_authentication,
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                            "authentication": {
                                "enable": "{{ True if authentication is defined and opt is undefined }}",
                                "message_digest": "{{ True if opt == 'message-digest' else None }}",
                                "null_auth": "{{ True if opt == 'null' else None }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "authentication.key_chain",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip)
                \sospf
                \s(?P<authentication>authentication)
                \skey-chain\s(?P<key_chain>\S+)$""",
                re.VERBOSE,
            ),
            "setval": "ip ospf authentication key-chain {{ authentication.key_chain }}",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ afi|replace('ip', 'ipv4') }}",
                            "authentication": {
                                "key_chain": "{{ key_chain }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "authentication_key",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip)
                \sospf
                \sauthentication-key
                \s(?P<encryption>\d)
                \s(?P<key>\S+)$""",
                re.VERBOSE,
            ),
            "setval": "ip ospf authentication-key "
                      "{{ authentication_key.encryption }} {{ authentication_key.key }}",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ afi|replace('ip', 'ipv4') }}",
                            "authentication_key": {
                                "encryption": "{{ encryption }}",
                                "key": "{{ key }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "message_digest_key",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip)
                \sospf
                \smessage-digest-key
                \s(?P<key_id>\d+)
                \smd5
                \s(?P<encryption>\d)
                \s(?P<key>\S+)$""",
                re.VERBOSE,
            ),
            "setval": "ip ospf "
                      "message-digest-key {{ message_digest_key.key_id }} "
                      "md5 {{ message_digest_key.encryption|default('') }} {{ message_digest_key.key }}",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi }}": {
                            "afi": "{{ afi|replace('ip', 'ipv4') }}",
                            "message_digest_key": {
                                "key_id": "{{ key_id }}",
                                "encryption": "{{ encryption }}",
                                "key": "{{ key }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "cost",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip)?
                \s(ospf|ospfv3)
                \scost\s(?P<cost>\d+)$""",
                re.VERBOSE,
            ),
            "setval": "{{ 'ip ' if afi == 'ipv4' else '' }}"
                      "{{ 'ospf' if afi == 'ipv4' else 'ospfv3' }} "
                      "cost {{ cost }}",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi|d('ipv6') }}": {
                            "afi": "{{ 'ipv4' if afi is defined else 'ipv6' }}",
                            "cost": "{{ cost }}",
                        },
                    },
                },
            },
        },
        {
            "name": "dead_interval",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip)?
                \s(ospf|ospfv3)
                \sdead-interval\s(?P<dead_interval>\d+)$""",
                re.VERBOSE,
            ),
            "setval": "{{ 'ip ' if afi == 'ipv4' else '' }}"
                      "{{ 'ospf' if afi == 'ipv4' else 'ospfv3' }} "
                      "dead-interval {{ dead_interval }}",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi|d('ipv6') }}": {
                            "afi": "{{ 'ipv4' if afi is defined else 'ipv6' }}",
                            "dead_interval": "{{ dead_interval }}",
                        },
                    },
                },
            },
        },
        {
            "name": "hello_interval",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip)?
                \s(ospf|ospfv3)
                \shello-interval\s(?P<hello_interval>\d+)$""",
                re.VERBOSE,
            ),
            "setval": "{{ 'ip ' if afi == 'ipv4' else '' }}"
                      "{{ 'ospf' if afi == 'ipv4' else 'ospfv3' }} "
                      "hello-interval {{ hello_interval }}",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi|d('ipv6') }}": {
                            "afi": "{{ 'ipv4' if afi is defined else 'ipv6' }}",
                            "hello_interval": "{{ hello_interval }}",
                        },
                    },
                },
            },
        },
        {
            "name": "instance",
            "getval": re.compile(
                r"""
                \s+(ospf|ospfv3)
                \sinstance\s(?P<instance>\d+)$""",
                re.VERBOSE,
            ),
            "setval": "ospfv3 instance {{ instance }}",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi|d('ipv6') }}": {
                            "afi": "ipv6",
                            "instance": "{{ instance }}",
                        },
                    },
                },
            },
        },
        {
            "name": "mtu_ignore",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip)?
                \s(ospf|ospfv3)
                \s(?P<mtu_ignore>mtu-ignore)$""",
                re.VERBOSE,
            ),
            "setval": "{{ 'ip ' if afi == 'ipv4' else '' }}"
                      "{{ 'ospf' if afi == 'ipv4' else 'ospfv3' }} "
                      "mtu-ignore",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi|d('ipv6') }}": {
                            "afi": "{{ 'ipv4' if afi is defined else 'ipv6' }}",
                            "mtu_ignore": "{{ not not mtu_ignore }}",
                        },
                    },
                },
            },
        },
        {
            "name": "network",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip)?
                \s(ospf|ospfv3)
                \snetwork\s(?P<network>(broadcast|point-to-point))$""",
                re.VERBOSE,
            ),
            "setval": "{{ 'ip ' if afi == 'ipv4' else '' }}"
                      "{{ 'ospf' if afi == 'ipv4' else 'ospfv3' }} "
                      "network {{ network }}",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi|d('ipv6') }}": {
                            "afi": "{{ 'ipv4' if afi is defined else 'ipv6' }}",
                            "network": "{{ network }}",
                        },
                    },
                },
            },
        },
        {
            "name": "passive_interface",
            "getval": re.compile(
                r"""
                (\s+(?P<negated>no))?
                (\s+(?P<afi>ip))?
                \s*(ospf|ospfv3)
                \s(?P<passive_interface>passive-interface)$""",
                re.VERBOSE,
            ),
            "setval": "{{ 'ip ' if afi == 'ipv4' else '' }}"
                      "{{ 'ospf' if afi == 'ipv4' else 'ospfv3' }} "
                      "passive-interface",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi|d('ipv6') }}": {
                            "afi": "{{ 'ipv4' if afi is defined else 'ipv6' }}",
                            "passive_interface": "{{ False if negated is defined else (not not passive_interface) }}",
                        },
                    },
                },
            },
        },
        {
            "name": "priority",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip)?
                \s(ospf|ospfv3)
                \spriority\s(?P<priority>\d+)$""",
                re.VERBOSE,
            ),
            "setval": "{{ 'ip ' if afi == 'ipv4' else '' }}"
                      "{{ 'ospf' if afi == 'ipv4' else 'ospfv3' }} "
                      "priority {{ priority }}",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi|d('ipv6') }}": {
                            "afi": "{{ 'ipv4' if afi is defined else 'ipv6' }}",
                            "priority": "{{ priority }}",
                        },
                    },
                },
            },
        },
        {
            "name": "retransmit_interval",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip)?
                \s(ospf|ospfv3)
                \sretransmit-interval\s(?P<retransmit_interval>\d+)$""",
                re.VERBOSE,
            ),
            "setval": "{{ 'ip ' if afi == 'ipv4' else '' }}"
                      "{{ 'ospf' if afi == 'ipv4' else 'ospfv3' }} "
                      "retransmit-interval {{ retransmit_interval }}",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi|d('ipv6') }}": {
                            "afi": "{{ 'ipv4' if afi is defined else 'ipv6' }}",
                            "retransmit_interval": "{{ retransmit_interval }}",
                        },
                    },
                },
            },
        },
        {
            "name": "shutdown",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip)?
                \s(ospf|ospfv3)
                \s(?P<shutdown>shutdown)$""",
                re.VERBOSE,
            ),
            "setval": "{{ 'ip ' if afi == 'ipv4' else '' }}"
                      "{{ 'ospf' if afi == 'ipv4' else 'ospfv3' }} "
                      "shutdown",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi|d('ipv6') }}": {
                            "afi": "{{ 'ipv4' if afi is defined else 'ipv6' }}",
                            "shutdown": "{{ not not shutdown }}",
                        },
                    },
                },
            },
        },
        {
            "name": "transmit_delay",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip)?
                \s(ospf|ospfv3)
                \stransmit-delay\s(?P<transmit_delay>\d+)$""",
                re.VERBOSE,
            ),
            "setval": "{{ 'ip ' if afi == 'ipv4' else '' }}"
                      "{{ 'ospf' if afi == 'ipv4' else 'ospfv3' }} "
                      "transmit-delay {{ transmit_delay }}",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ afi|d('ipv6') }}": {
                            "afi": "{{ 'ipv4' if afi is defined else 'ipv6' }}",
                            "transmit_delay": "{{ transmit_delay }}",
                        },
                    },
                },
            },
        },
    ]
    # fmt: on



#!/usr/bin/python
# coding: utf-8 -*-

# (c) 2017, Wayne Witzel III <wayne@riotousliving.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1', 'status': ['preview'], 'supported_by': 'community'}


DOCUMENTATION = '''
---
module: host
author: "Wayne Witzel III (@wwitzel3)"
short_description: create, update, or destroy Automation Platform Controller host.
description:
    - Create, update, or destroy Automation Platform Controller hosts. See
      U(https://www.ansible.com/tower) for an overview.
options:
    name:
      description:
        - The name to use for the host.
      required: True
      type: str
    new_name:
      description:
        - To use when changing a hosts's name.
      type: str
    description:
      description:
        - The description to use for the host.
      type: str
    inventory:
      description:
        - Inventory name, ID, or named URL the host should be made a member of.
      required: True
      type: str
    enabled:
      description:
        - If the host should be enabled.
      type: bool
    variables:
      description:
        - Variables to use for the host.
      type: dict
    state:
      description:
        - Desired state of the resource.
      choices: ["present", "absent", "exists"]
      default: "present"
      type: str
extends_documentation_fragment: awx.awx.auth
'''


EXAMPLES = '''
- name: Add host
  host:
    name: localhost
    description: "Local Host Group"
    inventory: "Local Inventory"
    state: present
    controller_config_file: "~/tower_cli.cfg"
    variables:
      example_var: 123
'''


from ..module_utils.controller_api import ControllerAPIModule
import json


def main():
    # Any additional arguments that are not fields of the item can be added here
    argument_spec = dict(
        name=dict(required=True),
        new_name=dict(),
        description=dict(),
        inventory=dict(required=True),
        enabled=dict(type='bool'),
        variables=dict(type='dict'),
        state=dict(choices=['present', 'absent', 'exists'], default='present'),
    )

    # Create a module for ourselves
    module = ControllerAPIModule(argument_spec=argument_spec)

    # Extract our parameters
    name = module.params.get('name')
    new_name = module.params.get('new_name')
    description = module.params.get('description')
    inventory = module.params.get('inventory')
    enabled = module.params.get('enabled')
    state = module.params.get('state')
    variables = module.params.get('variables')

    # Attempt to look up the related items the user specified (these will fail the module if not found)
    inventory_id = module.resolve_name_to_id('inventories', inventory)

    # Attempt to look up host based on the provided name and inventory ID
    host = module.get_one('hosts', name_or_id=name, check_exists=(state == 'exists'), **{'data': {'inventory': inventory_id}})

    if state == 'absent':
        # If the state was absent we can let the module delete it if needed, the module will handle exiting from this
        module.delete_if_needed(host)

    # Create the data that gets sent for create and update
    host_fields = {
        'name': new_name if new_name else (module.get_item_name(host) if host else name),
        'inventory': inventory_id,
        'enabled': enabled,
    }
    if description is not None:
        host_fields['description'] = description
    if variables is not None:
        host_fields['variables'] = json.dumps(variables)

    # If the state was present and we can let the module build or update the existing host, this will return on its own
    module.create_or_update_if_needed(host, host_fields, endpoint='hosts', item_type='host')


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = r"""
---
module: digital_ocean_project
short_description: Manage a DigitalOcean project
description:
     - Manage a project in DigitalOcean
author: "Tyler Auerbeck (@tylerauerbeck)"
version_added: 1.6.0

options:
  state:
    description:
      - Indicate desired state of the target.
      - C(present) will create the project
      - C(absent) will delete the project, if it exists.
    default: present
    choices: ['present', 'absent']
    type: str
  oauth_token:
    description:
      - DigitalOcean OAuth token. Can be specified in C(DO_API_KEY), C(DO_API_TOKEN), or C(DO_OAUTH_TOKEN) environment variables
    aliases: ['API_TOKEN']
    type: str
    required: true
  environment:
    description:
      - The environment of the projects resources.
    choices: ['Development', 'Staging', 'Production']
    type: str
  is_default:
    description:
      - If true, all resources will be added to this project if no project is specified.
    default: False
    type: bool
  name:
    description:
      - The human-readable name for the project. The maximum length is 175 characters and the name must be unique.
    type: str
  id:
    description:
      - UUID of the project
    type: str
  purpose:
    description:
      - The purpose of the project. The maximum length is 255 characters
      - Required if state is C(present)
      - If not one of DO provided purposes, will be prefixed with C(Other)
      - DO provided purposes can be found below
      - C(Just trying out DigitalOcean)
      - C(Class project/Educational Purposes)
      - C(Website or blog)
      - C(Web Application)
      - C(Service or API)
      - C(Mobile Application)
      - C(Machine Learning/AI/Data Processing)
      - C(IoT)
      - C(Operational/Developer tooling)
    type: str
  description:
    description:
      - The description of the project. The maximum length is 255 characters.
    type: str
"""


EXAMPLES = r"""
# Creates a new project
- community.digitalocean.digital_ocean_project:
    name: "TestProj"
    state: "present"
    description: "This is a test project"
    purpose: "IoT"
    environment: "Development"

# Updates the existing project with the new environment
- community.digitalocean.digital_ocean_project:
    name: "TestProj"
    state: "present"
    description: "This is a test project"
    purpose: "IoT"
    environment: "Production"

# This renames an existing project by utilizing the id of the project
- community.digitalocean.digital_ocean_project:
    name: "TestProj2"
    id: "12312312-abcd-efgh-ijkl-123123123123"
    state: "present"
    description: "This is a test project"
    purpose: "IoT"
    environment: "Development"

# This creates a project that results with a purpose of "Other: My Prod App"
- community.digitalocean.digital_ocean_project:
    name: "ProdProj"
    state: "present"
    description: "This is a prod app"
    purpose: "My Prod App"
    environment: "Production"

# This removes a project
- community.digitalocean.digital_ocean_project:
    name: "ProdProj"
    state: "absent"
"""

RETURN = r"""
# Digital Ocean API info https://docs.digitalocean.com/reference/api/api-reference/#tag/Projects
data:
    description: a DigitalOcean Project
    returned: changed
    type: dict
    sample: {
        "project": {
            "created_at": "2021-05-28T00:00:00Z",
            "description": "This is a test description",
            "environment": "Development",
            "id": "12312312-abcd-efgh-1234-abcdefgh123",
            "is_default": false,
            "name": "Test123",
            "owner_id": 1234567,
            "owner_uuid": "12312312-1234-5678-abcdefghijklm",
            "purpose": "IoT",
            "updated_at": "2021-05-29T00:00:00Z",
        }
    }
"""

from ansible.module_utils.basic import AnsibleModule, env_fallback
from ansible_collections.community.digitalocean.plugins.module_utils.digital_ocean import (
    DigitalOceanHelper,
)


class DOProject(object):
    def __init__(self, module):
        self.rest = DigitalOceanHelper(module)
        self.module = module
        # pop the oauth token so we don't include it in the POST data
        self.module.params.pop("oauth_token")
        self.id = None
        self.name = None
        self.purpose = None
        self.description = None
        self.environment = None
        self.is_default = None

    def get_by_id(self, project_id):
        if not project_id:
            return None
        response = self.rest.get("projects/{0}".format(project_id))
        json_data = response.json
        if response.status_code == 200:
            project = json_data.get("project", None)
            if project is not None:
                self.id = project.get("id", None)
                self.name = project.get("name", None)
                self.purpose = project.get("purpose", None)
                self.description = project.get("description", None)
                self.environment = project.get("environment", None)
                self.is_default = project.get("is_default", None)
            return json_data
        return None

    def get_by_name(self, project_name):
        if not project_name:
            return None
        page = 1
        while page is not None:
            response = self.rest.get("projects?page={0}".format(page))
            json_data = response.json
            if response.status_code == 200:
                for project in json_data["projects"]:
                    if project.get("name", None) == project_name:
                        self.id = project.get("id", None)
                        self.name = project.get("name", None)
                        self.description = project.get("description", None)
                        self.purpose = project.get("purpose", None)
                        self.environment = project.get("environment", None)
                        self.is_default = project.get("is_default", None)
                        return {"project": project}
                if (
                    "links" in json_data
                    and "pages" in json_data["links"]
                    and "next" in json_data["links"]["pages"]
                ):
                    page += 1
                else:
                    page = None
        return None

    def get_project(self):
        json_data = self.get_by_id(self.module.params["id"])
        if not json_data:
            json_data = self.get_by_name(self.module.params["name"])
        return json_data

    def create(self, state):
        json_data = self.get_project()
        request_params = dict(self.module.params)

        if json_data is not None:
            changed = False
            valid_purpose = [
                "Just trying out DigitalOcean",
                "Class project/Educational Purposes",
                "Website or blog",
                "Web Application",
                "Service or API",
                "Mobile Application",
                "Machine Learning/AI/Data Processing",
                "IoT",
                "Operational/Developer tooling",
            ]
            for key in request_params.keys():
                if (
                    key == "purpose"
                    and request_params[key] is not None
                    and request_params[key] not in valid_purpose
                ):
                    param = "Other: " + request_params[key]
                else:
                    param = request_params[key]

                if json_data["project"][key] != param and param is not None:
                    changed = True

            if changed:
                response = self.rest.put(
                    "projects/{0}".format(json_data["project"]["id"]),
                    data=request_params,
                )
                if response.status_code != 200:
                    self.module.fail_json(changed=False, msg="Unable to update project")
                self.module.exit_json(changed=True, data=response.json)
            else:
                self.module.exit_json(changed=False, data=json_data)
        else:
            response = self.rest.post("projects", data=request_params)

            if response.status_code != 201:
                self.module.fail_json(changed=False, msg="Unable to create project")
            self.module.exit_json(changed=True, data=response.json)

    def delete(self):
        json_data = self.get_project()
        if json_data:
            if self.module.check_mode:
                self.module.exit_json(changed=True)
            response = self.rest.delete(
                "projects/{0}".format(json_data["project"]["id"])
            )
            json_data = response.json
            if response.status_code == 204:
                self.module.exit_json(changed=True, msg="Project deleted")
            self.module.fail_json(changed=False, msg="Failed to delete project")
        else:
            self.module.exit_json(changed=False, msg="Project not found")


def core(module):
    state = module.params.pop("state")
    project = DOProject(module)
    if state == "present":
        project.create(state)
    elif state == "absent":
        project.delete()


def main():
    module = AnsibleModule(
        argument_spec=dict(
            state=dict(choices=["present", "absent"], default="present", type="str"),
            oauth_token=dict(
                aliases=["API_TOKEN"],
                no_log=True,
                fallback=(
                    env_fallback,
                    ["DO_API_TOKEN", "DO_API_KEY", "DO_OAUTH_TOKEN"],
                ),
                required=True,
            ),
            name=dict(type="str"),
            id=dict(type="str"),
            description=dict(type="str"),
            purpose=dict(type="str"),
            is_default=dict(type="bool", default=False),
            environment=dict(
                choices=["Development", "Staging", "Production"], type="str"
            ),
        ),
        required_one_of=(["id", "name"],),
        required_if=(
            [
                ("state", "present", ["purpose"]),
            ]
        ),
    )

    core(module)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: template_preview
short_description: Resource module for Template Preview
description:
- Manage operation update of the resource Template Preview.
- API to preview a template.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  deviceId:
    description: UUID of device to get template preview.
    type: str
  params:
    description: Params to render preview.
    type: dict
  resourceParams:
    description: Resource params to render preview.
    type: dict
  templateId:
    description: UUID of template to get template preview.
    type: str
requirements:
- dnacentersdk >= 2.5.5
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Configuration Templates PreviewTemplate
  description: Complete reference of the PreviewTemplate API.
  link: https://developer.cisco.com/docs/dna-center/#!preview-template
notes:
  - SDK Method used are
    configuration_templates.ConfigurationTemplates.preview_template,

  - Paths used are
    put /dna/intent/api/v1/template-programmer/template/preview,

"""

EXAMPLES = r"""
- name: Update all
  cisco.dnac.template_preview:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    deviceId: string
    params: {}
    resourceParams: {}
    templateId: string

"""

RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "cliPreview": "string",
      "deviceId": "string",
      "templateId": "string",
      "validationErrors": {}
    }
"""
